(function(angular, $, _) {

  angular.module('form_processor').config(function($routeProvider) {
      $routeProvider.when('/formProcessor/new', {
        templateUrl: '~/form_processor/FormProcessorEditCtrl.html',
        controller: 'FormProcessorEditCtrl',
        resolve: {
        	apiCalls: function($route, crmApi) {
        		reqs = {};
        		reqs.formProcessor = {
          		name: '',
          		title: '',
          		is_active: '1',
          		inputs: [],
          		actions: [],
          		output_handler: false,
          		output_handler_configuration: [],
          		enable_default_data: '0',
          		default_data_output_configuration: {},
          		default_data_inputs: [],
          		default_data_actions: [],
        		};
          	return reqs;
          }
        }
      });
      $routeProvider.when('/formProcessor/:id', {
        templateUrl: '~/form_processor/FormProcessorEditCtrl.html',
        controller: 'FormProcessorEditCtrl',
        resolve: {
        	apiCalls: function($route, crmApi) {
        		var reqs = {};
            if ($route.current.params.id !== 'new') {
              reqs.formProcessor = ['FormProcessorInstance', 'getsingle', {
                id: $route.current.params.id
              }];
            }
            return crmApi(reqs);
         	}
        }
      });
    }
  );
  
  angular.module('form_processor').controller('FormProcessorEditCtrl', function($scope, formProcessorFactory, dialogService, crmApi, apiCalls, $location, $q, $timeout) {
  	var ts = $scope.ts = CRM.ts(null);

    $scope.formProcessor = apiCalls.formProcessor;
    $scope.locks = {name: true};
    $scope.isNameValid = false;
    $scope.new_id = -1;
    $scope.deletedInputs = [];
    $scope.deletedActions = [];
    $scope.deletedDefaultDataInputs = [];
    $scope.deletedDefaultDataActions = [];
    $scope.default_data_fields = [];
    $scope.permissions = CRM.form_processor.permissions;
    $scope.actionTitles = CRM.form_processor.actionTitles;
    $scope.defaultDataActionTitles = CRM.form_processor.defaultDataActionTitles;
    
    $scope.outputHandlers = CRM.form_processor.outputHandlers;
    if (!$scope.formProcessor.output_handler) {
    	$scope.formProcessor.output_handler = CRM.form_processor.defaultOutputHandler;
    }
    
    // Convert to an object. Otherwise saving the configuration will fail because of a mix up between
    // object and array type. 
    if ($scope.formProcessor.default_data_output_configuration instanceof Array) {
    	$scope.formProcessor.default_data_output_configuration = {};
    }
    
    $scope.$watch('formProcessor.name', function(newFormProcessorName, oldFrmProcessorName) {
    	// Watch for changes in the name field
      crmApi('FormProcessorInstance', 'validatename', {'name': newFormProcessorName,'id': $scope.formProcessor.id})
      .then(function(data) {
      	$scope.isNameValid = false;
      	if (data.is_valid) {
      		$scope.isNameValid = true;
      	}
      });
    }, true);
    
    $scope.$watch('formProcessor.title', function(newFormProcessorTitle, oldFrmProcessorTitle) {
    	// Watch for changes in the name field
    	if ($scope.locks.name && !$scope.formProcessor.id) {
    		$scope.formProcessor.name = newFormProcessorTitle; 
    		$scope.formProcessor.name = $scope.formProcessor.name.toLowerCase();
    		$scope.formProcessor.name = $scope.formProcessor.name.replace(/[^a-zA-Z\d]/g, '_');
    		$scope.formProcessor.name = $scope.formProcessor.name.replace(/_+/g, '_');
    	}
    }, true);
    
    // Watch for changes in the default data actions if changed we need to populate the available fields again.
    $scope.$watchCollection('formProcessor.default_data_actions', function (newActions, oldActions) {
    	 formProcessorFactory.getFieldList([], $scope.formProcessor.default_data_actions, false, "form_processor_default_data")
    	 .then(function(fields) {
    	   $scope.default_data_fields = fields;
    	 });
    });
    
    $scope.isValidName = function(name) {
    	return (!name) || name.match(/^[a-z0-9_]+$/) ? true : false;
    };
    
    $scope.save = function(goBack) {    	
    	var output_handler = angular.copy($scope.formProcessor.output_handler);
    	$scope.formProcessor.output_handler = output_handler.name;
    	$scope.formProcessor.output_handler_configuration = output_handler.configuration;
      var result = crmApi('FormProcessorInstance', 'create', $scope.formProcessor, true);
      result.then(function(data) {
        $scope.formProcessor.id = data.id;
        $scope.formProcessor.output_handler = output_handler;  
        
        $scope.saveInputs($scope.formProcessor.inputs, $scope.deletedInputs, 'FormProcessorInput', $scope.formProcessor);
        $scope.saveInputs($scope.formProcessor.default_data_inputs, $scope.deletedDefaultDataInputs, 'FormProcessorDefaultDataInput', $scope.formProcessor);                
        $scope.saveActions($scope.formProcessor.actions, $scope.deletedActions, 'FormProcessorAction', $scope.formProcessor);
        $scope.saveActions($scope.formProcessor.default_data_actions, $scope.deletedDefaultDataActions, 'FormProcessorDefaultDataAction', $scope.formProcessor);
        
        $scope.editFormProcessorForm.$setPristine();
        
        if (goBack) {
        	$scope.goback();
        }
      });
    };
    
    $scope.goback = function () {
    	$location.path('/formprocessors');
    };
    
    $scope.saveInputs = function(inputs, deletedInputs, api, formProcessor) {
    	angular.forEach(deletedInputs, function(input, key) {
      	if (input.id) {
      		crmApi(api, 'delete', {'id': input.id}, true);
      	}
      });
      
      angular.forEach(inputs, function(input, key) {
      	if (input.id < 1) {
      		delete input.id;
      	}
      	var type = angular.copy(input.type);
      	input.form_processor_instance_id = formProcessor.id; 
      	input.type = input.type.name;
      	crmApi(api, 'create', input, true).then(function (input_result){
    			input.id = input_result['id'];
    			input.type = type;
    			
    			if (input.deletedValidators) {
    				angular.forEach(input.deletedValidators, function (validator, validator_index) {
    					if (validator.id) {
    						crmApi('FormProcessorValidator', 'delete', {'id': id});
    					}	
  					});
					}
    			
    			angular.forEach(input.validators, function(validator, validator_index){
    				validator.entity = api;
    				validator.entity_id = input.id;
    				var validator_type = angular.copy(validator.validator);
    				validator.validator = validator.validator.name;
    				crmApi('FormProcessorValidation', 'create', validator).then(function (validator_result) {
    					validator.validator = validator_type;
    				});
    			});
    		});
      });
    };
    
    $scope.saveActions = function(actions, deletedActions, api, formProcessor) {
    	angular.forEach(deletedActions, function(action, key) {
      	if (action.id) {          	
      		crmApi(api, 'delete', {'id': action.id}, true);
      	}
      });
      
      var actionWeight = 0;
      angular.forEach(actions, function(action, key) {
      	var old_id = action.id;
      	if (action.id < 0) {
      		delete action.id;
      	}
      	action.form_processor_instance_id = formProcessor.id;
      	action.weight = actionWeight;
      	crmApi(api, 'create', action, true).then(function(action_result) {
      		action.id = action_result.id;
    		});
      	actionWeight++;
      });
    };
    
    $scope.editOutputHandler = function editOutputHandler() {
    	var options = CRM.utils.adjustDialogDefaults({
        autoOpen: false,
        width: '40%',
        height: 'auto',
        title: ts('Edit output handler')
      });
      
      formProcessorFactory.getFieldList($scope.formProcessor.inputs, $scope.formProcessor.actions, false, 'form_processor')
      .then(function (fields) {
        var model = {
      	 formProcessor: $scope.formProcessor,
      	 outputHandler: $scope.formProcessor.output_handler,
      	 outputHandlers: $scope.outputHandlers,
      	 fields: fields
        };
        dialogService.open('OutputHandlerDialog', '~/form_processor/OutputHandlerDialogCtrl.html', model, options)
        .then(function(data) {
      	  $scope.editFormProcessorForm.$setDirty();
      	  $scope.formProcessor.output_handler = data.outputHandler;
			  });
		  });
    };
  	
  });

})(angular, CRM.$, CRM._);
